//+------------------------------------------------------------------+
//|                                            FiboCandlesSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "FiboCandles.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    FiboCandles              |
//| Type=SignalAdvanced                                              |
//| Name=FiboCandles                                                 |
//| Class=CFiboCandlesSignal                                         |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=period,uint,10,                        |
//| Parameter=fiboLevel,uint,1,  (1 - 5)           |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CFiboCandlesSignal.                                        |
//| :          |
//| FiboCandles http://www.mql5.com/ru/code/536/.                    |
//|                 CExpertSignal.        |
//+------------------------------------------------------------------+
class CFiboCandlesSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      FiboCandles

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //       
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   FiboCandles
   uint              m_period;           //  
   uint              m_fiboLevel;        //   (1 - 5) 
   uint              m_SignalBar;        //       
   
public:
                     CFiboCandlesSignal();
   //---    
   void              BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void              SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void              BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void              SellPosClose(bool value)                { m_SellPosClose=value;     }
   void              Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void              period(uint value)                      { m_period=value;           }
   void              fiboLevel(uint value)                   { m_fiboLevel=value;        }
   void              SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitFiboCandles(CIndicators *indicators);   // ,   FiboCandles
  };
//+------------------------------------------------------------------+
//|  CFiboCandlesSignal.                                  |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CFiboCandlesSignal::CFiboCandlesSignal()
  {
//---   -
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;
   m_Ind_Timeframe=PERIOD_H4; 
   m_period=10;
   m_fiboLevel=1; 
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CFiboCandlesSignal::ValidationSettings()
  {
//---  
   if(m_period<=0)
     {
      printf(__FUNCTION__+":      !");
      printf(__FUNCTION__+":       1!");
      m_period=1;
     }
     
   if(m_fiboLevel<1)
     {
      printf(__FUNCTION__+":        1!");
      printf(__FUNCTION__+":       1!");
      m_fiboLevel=1;
     }

   if(m_fiboLevel>5)
     {
      printf(__FUNCTION__+":        5!");
      printf(__FUNCTION__+":       5!");
      m_fiboLevel=5;
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CFiboCandlesSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitFiboCandles(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   FiboCandles.                            |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CFiboCandlesSignal::InitFiboCandles(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[5];
   
   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="FiboCandles.ex5";
   
   parameters[1].type=TYPE_UINT;
   parameters[1].integer_value=m_period;
   
   parameters[2].type=TYPE_UINT;
   parameters[2].integer_value=m_fiboLevel;
   
   parameters[3].type=TYPE_UINT;
   parameters[3].integer_value=m_SignalBar;
   
   parameters[4].type=TYPE_UINT;
   parameters[4].integer_value=0;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,5,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
     
//---  
   if(!m_indicator.NumBuffers(5)) return(false);
   
//---     FiboCandles
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CFiboCandlesSignal::LongCondition()
  {
//---       4  FiboCandles
   double Signal0=m_indicator.GetData(4,m_SignalBar);
   double Signal1=m_indicator.GetData(4,m_SignalBar+1);

//---    
   if(Signal0==0)
     {
      if(Signal1==1)
        {
         if(m_BuyPosOpen)
           {
            if(m_SellPosClose) return(REVERSE_SHORT);
            else return(OPEN_LONG);
           }
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }
//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CFiboCandlesSignal::ShortCondition()
  {
//---       4  FiboCandles
   double Signal0=m_indicator.GetData(4,m_SignalBar);
   double Signal1=m_indicator.GetData(4,m_SignalBar+1);

//---    
   if(Signal0==1)
     {
      if(Signal1==0)
        {
         if(m_SellPosOpen)
           {
            if(m_BuyPosClose) return(REVERSE_LONG);
            else return(OPEN_SHORT);
           }
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }
//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+